@@ -1,6 +1,8 @@
 __all__ = ['utils', 'types', 'meta', 'lang', 'tree_mini', 'parsing', 'magics',
-           'lnsetup', 'display', 'combinators']
+           'lnsetup', 'display', 'combinators', 'auto']
 
-__version_info__ = (0, 6, 9)
+__version_info__ = (0, 7, 0)
 __release__ = True
 __version__ = ".".join(str(i) for i in __version_info__)
+if not __release__:
+    __version__ = __version__ + "a1"
